@@ -21,14 +21,16 @@ package org.apache.polaris.service.storage.aws;
 import static org.assertj.core.api.Assertions.assertThat;
 
 import jakarta.annotation.Nonnull;
+import java.time.Instant;
 import java.util.EnumMap;
 import java.util.List;
 import java.util.Set;
 import org.apache.polaris.core.PolarisDiagnostics;
-import org.apache.polaris.core.storage.PolarisCredentialProperty;
 import org.apache.polaris.core.storage.PolarisStorageConfigurationInfo;
+import org.apache.polaris.core.storage.StorageAccessProperty;
 import org.apache.polaris.core.storage.aws.AwsCredentialsStorageIntegration;
 import org.apache.polaris.core.storage.aws.AwsStorageConfigurationInfo;
+import org.assertj.core.api.Assertions;
 import org.assertj.core.api.InstanceOfAssertFactories;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.params.ParameterizedTest;
@@ -48,6 +50,8 @@ import software.amazon.awssdk.services.sts.model.Credentials;
 
 class AwsCredentialsStorageIntegrationTest {
 
+  public static final Instant EXPIRE_TIME = Instant.now().plusMillis(3600_000);
+
   public static final AssumeRoleResponse ASSUME_ROLE_RESPONSE =
       AssumeRoleResponse.builder()
           .credentials(
@@ -55,6 +59,7 @@ class AwsCredentialsStorageIntegrationTest {
                   .accessKeyId("accessKey")
                   .secretAccessKey("secretKey")
                   .sessionToken("sess")
+                  .expiration(EXPIRE_TIME)
                   .build())
           .build();
   public static final String AWS_PARTITION = "aws";
@@ -75,7 +80,7 @@ class AwsCredentialsStorageIntegrationTest {
               return ASSUME_ROLE_RESPONSE;
             });
     String warehouseDir = "s3://bucket/path/to/warehouse";
-    EnumMap<PolarisCredentialProperty, String> credentials =
+    EnumMap<StorageAccessProperty, String> credentials =
         new AwsCredentialsStorageIntegration(stsClient)
             .getSubscopedCreds(
                 Mockito.mock(PolarisDiagnostics.class),
@@ -90,9 +95,12 @@ class AwsCredentialsStorageIntegrationTest {
                 Set.of(warehouseDir + "/namespace/table"));
     assertThat(credentials)
         .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.AWS_TOKEN, "sess")
-        .containsEntry(PolarisCredentialProperty.AWS_KEY_ID, "accessKey")
-        .containsEntry(PolarisCredentialProperty.AWS_SECRET_KEY, "secretKey");
+        .containsEntry(StorageAccessProperty.AWS_TOKEN, "sess")
+        .containsEntry(StorageAccessProperty.AWS_KEY_ID, "accessKey")
+        .containsEntry(StorageAccessProperty.AWS_SECRET_KEY, "secretKey")
+        .containsEntry(
+            StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS,
+            String.valueOf(EXPIRE_TIME.toEpochMilli()));
   }
 
   @ParameterizedTest
@@ -101,15 +109,19 @@ class AwsCredentialsStorageIntegrationTest {
     PolarisStorageConfigurationInfo.StorageType storageType =
         PolarisStorageConfigurationInfo.StorageType.S3;
     String roleARN;
+    String region;
     switch (awsPartition) {
       case AWS_PARTITION:
         roleARN = "arn:aws:iam::012345678901:role/jdoe";
+        region = "us-east-1";
         break;
       case "aws-cn":
         roleARN = "arn:aws-cn:iam::012345678901:role/jdoe";
+        region = "Beijing";
         break;
       case "aws-us-gov":
         roleARN = "arn:aws-us-gov:iam::012345678901:role/jdoe";
+        region = "us-gov-west-1";
         break;
       default:
         throw new IllegalArgumentException("Unknown aws partition: " + awsPartition);
@@ -213,24 +225,51 @@ class AwsCredentialsStorageIntegrationTest {
                       });
               return ASSUME_ROLE_RESPONSE;
             });
-    EnumMap<PolarisCredentialProperty, String> credentials =
-        new AwsCredentialsStorageIntegration(stsClient)
-            .getSubscopedCreds(
-                Mockito.mock(PolarisDiagnostics.class),
-                new AwsStorageConfigurationInfo(
-                    storageType,
-                    List.of(s3Path(bucket, warehouseKeyPrefix)),
-                    roleARN,
-                    externalId,
-                    "us-east-2"),
-                true,
-                Set.of(s3Path(bucket, firstPath), s3Path(bucket, secondPath)),
-                Set.of(s3Path(bucket, firstPath)));
-    assertThat(credentials)
-        .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.AWS_TOKEN, "sess")
-        .containsEntry(PolarisCredentialProperty.AWS_KEY_ID, "accessKey")
-        .containsEntry(PolarisCredentialProperty.AWS_SECRET_KEY, "secretKey");
+    switch (awsPartition) {
+      case "aws-cn":
+        Assertions.assertThatThrownBy(
+                () ->
+                    new AwsCredentialsStorageIntegration(stsClient)
+                        .getSubscopedCreds(
+                            Mockito.mock(PolarisDiagnostics.class),
+                            new AwsStorageConfigurationInfo(
+                                storageType,
+                                List.of(s3Path(bucket, warehouseKeyPrefix)),
+                                roleARN,
+                                externalId,
+                                region),
+                            true,
+                            Set.of(s3Path(bucket, firstPath), s3Path(bucket, secondPath)),
+                            Set.of(s3Path(bucket, firstPath))))
+            .isInstanceOf(IllegalArgumentException.class);
+        break;
+      case AWS_PARTITION:
+      case "aws-us-gov":
+        EnumMap<StorageAccessProperty, String> credentials =
+            new AwsCredentialsStorageIntegration(stsClient)
+                .getSubscopedCreds(
+                    Mockito.mock(PolarisDiagnostics.class),
+                    new AwsStorageConfigurationInfo(
+                        storageType,
+                        List.of(s3Path(bucket, warehouseKeyPrefix)),
+                        roleARN,
+                        externalId,
+                        region),
+                    true,
+                    Set.of(s3Path(bucket, firstPath), s3Path(bucket, secondPath)),
+                    Set.of(s3Path(bucket, firstPath)));
+        assertThat(credentials)
+            .isNotEmpty()
+            .containsEntry(StorageAccessProperty.AWS_TOKEN, "sess")
+            .containsEntry(StorageAccessProperty.AWS_KEY_ID, "accessKey")
+            .containsEntry(StorageAccessProperty.AWS_SECRET_KEY, "secretKey")
+            .containsEntry(
+                StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS,
+                String.valueOf(EXPIRE_TIME.toEpochMilli()));
+        break;
+      default:
+        throw new IllegalArgumentException("Unknown aws partition: " + awsPartition);
+    }
   }
 
   @Test
@@ -307,7 +346,7 @@ class AwsCredentialsStorageIntegrationTest {
             });
     PolarisStorageConfigurationInfo.StorageType storageType =
         PolarisStorageConfigurationInfo.StorageType.S3;
-    EnumMap<PolarisCredentialProperty, String> credentials =
+    EnumMap<StorageAccessProperty, String> credentials =
         new AwsCredentialsStorageIntegration(stsClient)
             .getSubscopedCreds(
                 Mockito.mock(PolarisDiagnostics.class),
@@ -322,9 +361,12 @@ class AwsCredentialsStorageIntegrationTest {
                 Set.of(s3Path(bucket, firstPath)));
     assertThat(credentials)
         .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.AWS_TOKEN, "sess")
-        .containsEntry(PolarisCredentialProperty.AWS_KEY_ID, "accessKey")
-        .containsEntry(PolarisCredentialProperty.AWS_SECRET_KEY, "secretKey");
+        .containsEntry(StorageAccessProperty.AWS_TOKEN, "sess")
+        .containsEntry(StorageAccessProperty.AWS_KEY_ID, "accessKey")
+        .containsEntry(StorageAccessProperty.AWS_SECRET_KEY, "secretKey")
+        .containsEntry(
+            StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS,
+            String.valueOf(EXPIRE_TIME.toEpochMilli()));
   }
 
   @Test
@@ -399,7 +441,7 @@ class AwsCredentialsStorageIntegrationTest {
             });
     PolarisStorageConfigurationInfo.StorageType storageType =
         PolarisStorageConfigurationInfo.StorageType.S3;
-    EnumMap<PolarisCredentialProperty, String> credentials =
+    EnumMap<StorageAccessProperty, String> credentials =
         new AwsCredentialsStorageIntegration(stsClient)
             .getSubscopedCreds(
                 Mockito.mock(PolarisDiagnostics.class),
@@ -414,9 +456,12 @@ class AwsCredentialsStorageIntegrationTest {
                 Set.of());
     assertThat(credentials)
         .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.AWS_TOKEN, "sess")
-        .containsEntry(PolarisCredentialProperty.AWS_KEY_ID, "accessKey")
-        .containsEntry(PolarisCredentialProperty.AWS_SECRET_KEY, "secretKey");
+        .containsEntry(StorageAccessProperty.AWS_TOKEN, "sess")
+        .containsEntry(StorageAccessProperty.AWS_KEY_ID, "accessKey")
+        .containsEntry(StorageAccessProperty.AWS_SECRET_KEY, "secretKey")
+        .containsEntry(
+            StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS,
+            String.valueOf(EXPIRE_TIME.toEpochMilli()));
   }
 
   @Test
@@ -461,7 +506,7 @@ class AwsCredentialsStorageIntegrationTest {
                       });
               return ASSUME_ROLE_RESPONSE;
             });
-    EnumMap<PolarisCredentialProperty, String> credentials =
+    EnumMap<StorageAccessProperty, String> credentials =
         new AwsCredentialsStorageIntegration(stsClient)
             .getSubscopedCreds(
                 Mockito.mock(PolarisDiagnostics.class),
@@ -476,15 +521,19 @@ class AwsCredentialsStorageIntegrationTest {
                 Set.of());
     assertThat(credentials)
         .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.AWS_TOKEN, "sess")
-        .containsEntry(PolarisCredentialProperty.AWS_KEY_ID, "accessKey")
-        .containsEntry(PolarisCredentialProperty.AWS_SECRET_KEY, "secretKey");
+        .containsEntry(StorageAccessProperty.AWS_TOKEN, "sess")
+        .containsEntry(StorageAccessProperty.AWS_KEY_ID, "accessKey")
+        .containsEntry(StorageAccessProperty.AWS_SECRET_KEY, "secretKey")
+        .containsEntry(
+            StorageAccessProperty.AWS_SESSION_TOKEN_EXPIRES_AT_MS,
+            String.valueOf(EXPIRE_TIME.toEpochMilli()));
   }
 
-  @Test
-  public void testClientRegion() {
+  @ParameterizedTest
+  @ValueSource(strings = {AWS_PARTITION, "aws-cn", "aws-us-gov"})
+  public void testClientRegion(String awsPartition) {
     StsClient stsClient = Mockito.mock(StsClient.class);
-    String roleARN = "arn:aws:iam::012345678901:role/jdoe";
+    String roleARN = "arn:aws:iam::012345678901:role/jdoe".replaceFirst("aws", awsPartition);
     String externalId = "externalId";
     String bucket = "bucket";
     String warehouseKeyPrefix = "path/to/warehouse";
@@ -494,28 +543,54 @@ class AwsCredentialsStorageIntegrationTest {
             invocation -> {
               return ASSUME_ROLE_RESPONSE;
             });
-    EnumMap<PolarisCredentialProperty, String> credentials =
-        new AwsCredentialsStorageIntegration(stsClient)
-            .getSubscopedCreds(
-                Mockito.mock(PolarisDiagnostics.class),
-                new AwsStorageConfigurationInfo(
-                    PolarisStorageConfigurationInfo.StorageType.S3,
-                    List.of(s3Path(bucket, warehouseKeyPrefix)),
-                    roleARN,
-                    externalId,
-                    clientRegion),
-                true, /* allowList = true */
-                Set.of(),
-                Set.of());
-    assertThat(credentials)
-        .isNotEmpty()
-        .containsEntry(PolarisCredentialProperty.CLIENT_REGION, clientRegion);
+    switch (awsPartition) {
+      case "aws-cn":
+        Assertions.assertThatThrownBy(
+                () ->
+                    new AwsCredentialsStorageIntegration(stsClient)
+                        .getSubscopedCreds(
+                            Mockito.mock(PolarisDiagnostics.class),
+                            new AwsStorageConfigurationInfo(
+                                PolarisStorageConfigurationInfo.StorageType.S3,
+                                List.of(s3Path(bucket, warehouseKeyPrefix)),
+                                roleARN,
+                                externalId,
+                                clientRegion),
+                            true, /* allowList = true */
+                            Set.of(),
+                            Set.of()))
+            .isInstanceOf(IllegalArgumentException.class);
+        break;
+      case AWS_PARTITION:
+      case "aws-us-gov":
+        EnumMap<StorageAccessProperty, String> credentials =
+            new AwsCredentialsStorageIntegration(stsClient)
+                .getSubscopedCreds(
+                    Mockito.mock(PolarisDiagnostics.class),
+                    new AwsStorageConfigurationInfo(
+                        PolarisStorageConfigurationInfo.StorageType.S3,
+                        List.of(s3Path(bucket, warehouseKeyPrefix)),
+                        roleARN,
+                        externalId,
+                        clientRegion),
+                    true, /* allowList = true */
+                    Set.of(),
+                    Set.of());
+        assertThat(credentials)
+            .isNotEmpty()
+            .containsEntry(StorageAccessProperty.CLIENT_REGION, clientRegion);
+        break;
+      default:
+        throw new IllegalArgumentException("Unknown aws partition: " + awsPartition);
+    }
+    ;
   }
 
-  @Test
-  public void testNoClientRegion() {
+  @ParameterizedTest
+  @ValueSource(strings = {AWS_PARTITION, "aws-cn", "aws-us-gov"})
+  public void testNoClientRegion(String awsPartition) {
     StsClient stsClient = Mockito.mock(StsClient.class);
-    String roleARN = "arn:aws:iam::012345678901:role/jdoe";
+    String roleARN = "arn:aws:iam::012345678901:role/jdoe".replaceFirst("aws", awsPartition);
     String externalId = "externalId";
     String bucket = "bucket";
     String warehouseKeyPrefix = "path/to/warehouse";
@@ -524,20 +599,45 @@ class AwsCredentialsStorageIntegrationTest {
             invocation -> {
               return ASSUME_ROLE_RESPONSE;
             });
-    EnumMap<PolarisCredentialProperty, String> credentials =
-        new AwsCredentialsStorageIntegration(stsClient)
-            .getSubscopedCreds(
-                Mockito.mock(PolarisDiagnostics.class),
-                new AwsStorageConfigurationInfo(
-                    PolarisStorageConfigurationInfo.StorageType.S3,
-                    List.of(s3Path(bucket, warehouseKeyPrefix)),
-                    roleARN,
-                    externalId,
-                    null),
-                true, /* allowList = true */
-                Set.of(),
-                Set.of());
-    assertThat(credentials).isNotEmpty().doesNotContainKey(PolarisCredentialProperty.CLIENT_REGION);
+    switch (awsPartition) {
+      case AWS_PARTITION:
+        EnumMap<StorageAccessProperty, String> credentials =
+            new AwsCredentialsStorageIntegration(stsClient)
+                .getSubscopedCreds(
+                    Mockito.mock(PolarisDiagnostics.class),
+                    new AwsStorageConfigurationInfo(
+                        PolarisStorageConfigurationInfo.StorageType.S3,
+                        List.of(s3Path(bucket, warehouseKeyPrefix)),
+                        roleARN,
+                        externalId,
+                        null),
+                    true, /* allowList = true */
+                    Set.of(),
+                    Set.of());
+        assertThat(credentials).isNotEmpty().doesNotContainKey(StorageAccessProperty.CLIENT_REGION);
+        break;
+      case "aws-cn":
+      case "aws-us-gov":
+        Assertions.assertThatThrownBy(
+                () ->
+                    new AwsCredentialsStorageIntegration(stsClient)
+                        .getSubscopedCreds(
+                            Mockito.mock(PolarisDiagnostics.class),
+                            new AwsStorageConfigurationInfo(
+                                PolarisStorageConfigurationInfo.StorageType.S3,
+                                List.of(s3Path(bucket, warehouseKeyPrefix)),
+                                roleARN,
+                                externalId,
+                                null),
+                            true, /* allowList = true */
+                            Set.of(),
+                            Set.of()))
+            .isInstanceOf(IllegalArgumentException.class);
+        break;
+      default:
+        throw new IllegalArgumentException("Unknown aws partition: " + awsPartition);
+    }
+    ;
   }
 
   private static @Nonnull String s3Arn(String partition, String bucket, String keyPrefix) {
